// non-human species vs. human

#include "main.h"
#include "read_maf.h"
#include "util.h"
#include "util_i.h"
#include "regions.h"
#include "util_input.h"
#include "util_gen.h"
#include "find_ps.h"

int debug_mode;
char S[BIG], T[BIG];
char S1[BIG], T1[BIG];

int main(int argc, char **argv)
{
	struct DotList *algns;
	int *num_algns;
	int *size1, *size2;
	int count = 0;
	char species[100], species2[100];
	char temp_name[100];
	FILE *f;
	struct exons_list *exons1, *exons2, *genes1, *genes2; 
	int *num_exons1, *num_genes1;
	int *num_exons2, *num_genes2;
	struct exons_list *candi_ps;
	int *num_candi, *num_alloc;
	int temp_num1 = 0, temp_num2 = 0;
	int i = 0, j = 0;
	int b = 0, e = 0;
	int lo = 0, hi = 1;

	debug_mode = FALSE;
	if( argc == 6 ) {
		debug_mode = TRUE;
	}
	else if( argc != 5 ) {
		fatal("args: $(species2).human.chained_maf human.codex $(species2).codex ref_sp\n");
	}

	size1 = (int *) ckalloc(sizeof(int));
	size2 = (int *) ckalloc(sizeof(int));
	num_algns = (int *) ckalloc(sizeof(int));
	*size1 = 0;
	*size2 = 0;
	*num_algns = 0;
	count = count_local_algns(argv[1], species, species2);
	if( count > 0 ) {
		algns = (struct DotList *) ckalloc(count * sizeof(struct DotList));
		initialize_algns(algns, 0, count);

	 	read_maf(argv[1], G_MODE, algns, num_algns, size1, size2);
	}

	if( count != (*num_algns) ) {
		fatalf("counting error 1: %d vs. %d\n", count, *num_algns);
	}

	num_exons1 = (int *) ckalloc(sizeof(int));
	num_exons2 = (int *) ckalloc(sizeof(int));
	*num_exons1 = 0;
	*num_exons2 = 0;
	num_genes1 = (int *) ckalloc(sizeof(int));
	num_genes2 = (int *) ckalloc(sizeof(int));
	*num_genes1 = 0;
	*num_genes2 = 0;

	count = count_genes(argv[2], species, species);
	genes1 = (struct exons_list *) ckalloc(count * sizeof(struct exons_list));

	if( count > 0 ) {
		initialize_exons_list(genes1, 0, count);
	}

	count = count_exons(argv[2], species, species);
	exons1 = (struct exons_list *) ckalloc(count * sizeof(struct exons_list));

	if( count > 0 ) {
		initialize_exons_list(exons1, 0, count);
		read_only_exons(exons1, num_exons1, genes1, num_genes1, argv[2], species, species);
	}

	if( count != (*num_exons1) ) {
		fatalf("counting error 3: %d vs. %d\n", count, *num_exons1);
	}

	count = count_genes(argv[3], species2, species2);
	genes2 = (struct exons_list *) ckalloc(count * sizeof(struct exons_list));

	if( count > 0 ) {
		initialize_exons_list(genes2, 0, count);
	}

	count = count_exons(argv[3], species2, species2);
	exons2 = (struct exons_list *) ckalloc(count * sizeof(struct exons_list));

	if( count > 0 ) {
		initialize_exons_list(exons2, 0, count);
		read_only_exons(exons2, num_exons2, genes2, num_genes2, argv[3], species2, species2);
	}

	if( count != (*num_exons2) ) {
		fatalf("counting error 3: %d vs. %d\n", count, *num_exons2);
	}

	f = ckopen(argv[1], "r");
  num_candi = (int *) ckalloc(sizeof(int));
  num_alloc = (int *) ckalloc(sizeof(int));
  *num_alloc = ALLOC_UNIT;
  *num_candi = 0;
  candi_ps = (struct exons_list *) ckalloc(ALLOC_UNIT * sizeof(struct exons_list));
  initialize_exons_list(candi_ps, 0, ALLOC_UNIT);

	find_pseudogenes(genes1, num_genes1, genes2, num_genes2, exons2, num_exons2, algns, *num_algns, f, &candi_ps, num_candi, num_alloc);

	if( (*num_candi) > 0 ) {
		sort_exons(candi_ps, *num_candi);
	}

	i = 0;
	while( i < (*num_candi) ) {
		if( candi_ps[i].sign != 'n' ) {
			j = i + 1;
			while( (j < (*num_candi)) && (candi_ps[j].sign != 'n') && (proper_overlap(candi_ps[i].reg, candi_ps[j].reg) == true) ) {
				if( candi_ps[i].reg.lower < candi_ps[j].reg.lower) lo = candi_ps[i].reg.lower;
				else lo = candi_ps[j].reg.lower;

				if( candi_ps[i].reg.upper < candi_ps[j].reg.upper) hi = candi_ps[j].reg.upper;
				else hi = candi_ps[i].reg.upper;
							
				candi_ps[i].reg = assign_I(lo, hi);
				candi_ps[j].sign = 'n';
				j++;
			}
			i = j;
		}
		else {
			i++;
		}
	}

	temp_num1 = *num_genes1;
	temp_num2 = *num_exons1;
	if( (*num_candi) > 0 ) {
		sort_exons(candi_ps, *num_candi);
		genes1 = ckrealloc(genes1, ((*num_genes1) + (*num_candi)) * sizeof(struct exons_list));
		exons1 = ckrealloc(exons1, ((*num_exons1) + (*num_candi)) * sizeof(struct exons_list));
		j = 1;
		for( i = 0; i < (*num_candi); i++ ) {
			if( candi_ps[i].sign != 'n' ) {
				sprintf(temp_name, "%s%d_ps", species2, j);
				j++;
				genes1[temp_num1].reg = assign_I(candi_ps[i].reg.lower, candi_ps[i].reg.upper);	
				genes1[temp_num1].sign = candi_ps[i].sign;
				exons1[temp_num2].reg = assign_I(candi_ps[i].reg.lower, candi_ps[i].reg.upper);	
				exons1[temp_num2].sign = candi_ps[i].sign;
				strcpy(genes1[temp_num1].name, temp_name);
				strcpy(exons1[temp_num2].name, temp_name);
				exons1[temp_num2].fid = temp_num1;
				temp_num1++;
				temp_num2++;
			}
		}
	}

	*num_genes1 = temp_num1;
	*num_exons1 = temp_num2;

	for( i = 0; i < temp_num2; i++ ) {
		j = exons1[i].fid;
		b = i;
		while( (i < (temp_num2-1)) && (exons1[i].fid == exons1[i+1].fid) ) {
			i++;
		}
		e = i;
		genes1[j].cmp_reg = assign_I(b, e); // cmp_reg saves index for start exon and end temporally
	}

	sort_exons(genes1, *num_genes1);
	j = 1;
	for( i = 0; i < (*num_genes1); i++ ) {
		if( (strstr(species2, "human") != NULL) || (strcmp(species2, argv[4]) == 0) ) {
			strcpy(temp_name, genes1[i].name);
		}
		else if( strstr(genes1[i].name, "_ps") != NULL ) {
			sprintf(temp_name, "%s%d_ps", species2, j);
			j++;
		}
		else {
			sprintf(temp_name, "%s%d", species2, j);
			j++;
		}
		
		if( genes1[i].sign == '<' ) {
			printf("%c %d %d %s (complement)\n", genes1[i].sign, genes1[i].reg.lower, genes1[i].reg.upper-1, temp_name);
		}
		else if( genes1[i].sign == '>') {
			printf("%c %d %d %s\n", genes1[i].sign, genes1[i].reg.lower, genes1[i].reg.upper-1, temp_name);
		}

		if( (genes1[i].sign == '<') || (genes1[i].sign == '>') ) {
			for( b = genes1[i].cmp_reg.lower; b <= genes1[i].cmp_reg.upper; b++ ) {
				printf("%d %d\n", exons1[b].reg.lower, exons1[b].reg.upper-1);
			}
		}
	}

	fclose(f);
	free(size1);
	free(size2);

	if( (*num_algns) > 0 ) {
		free(algns);
	}

	free(num_candi);
	free(num_alloc);
	free(candi_ps);
	free(num_algns);
	free(exons1);
	free(genes1);
	free(num_exons1);
	free(num_genes1);
	free(exons2);
	free(genes2);
	free(num_exons2);
	free(num_genes2);
	return(EXIT_SUCCESS);
}

